"""
# File       : apis.py
# Time       ：2024/8/26 下午10:19
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：
"""
from fastapi import APIRouter, HTTPException, Depends, Request
from fastapi.security import OAuth2PasswordBearer, OAuth2PasswordRequestForm
import httpx
from app_tools_zxw.msvc_user_auth.schemas import *

# OAuth2PasswordBearer 实例
oauth2_scheme = OAuth2PasswordBearer(tokenUrl="/user_center/account/login-form/")


# Pydantic 模型
class WeChatQRCodeRequest(BaseModel):
    WECHAT_REDIRECT_URI: str


class WeChatLoginRequest(BaseModel):
    code: str
    app_name: str


class TokenRefreshRequest(BaseModel):
    refresh_token: str


class RoleAuthRequest(BaseModel):
    role_name: str
    app_name: str


class InterfaceUserAuth:
    def __init__(self, svc_user: str = "http://127.0.0.1:8101"):
        self.svc_user = svc_user

    async def 账号密码_注册(self, data: 请求_账号密码_注册) -> 返回_login:
        # 调用用户管理微服务进行普通注册
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/api/account/register/",
                json=data.model_dump()
            )
            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to register")
            return response.json()

    async def 账号密码_登录(self, data: 请求_账号密码_登录) -> 返回_login:
        # 调用用户管理微服务进行普通登录
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/api/account/login/",
                json=data.model_dump()
            )
            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to login")
            return response.json()

    async def 获取_登录二维码URL(self, request: WeChatQRCodeRequest) -> 返回_获取_登录二维码URL:
        # 调用用户管理微服务获取微信二维码URL
        # 请求URL DEMO ： http://127.0.0.1:8101/wechat/qr-login/get-qrcode
        print({"WECHAT_REDIRECT_URI": request.WECHAT_REDIRECT_URI})
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/wechat/qr-login/get-qrcode",
                json={"WECHAT_REDIRECT_URI": request.WECHAT_REDIRECT_URI}
            )
            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to get QR code")
            return response.json()

    async def 微信登录(self, request: WeChatLoginRequest) -> 返回_login:
        # 调用用户管理微服务进行微信登录
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/wechat/qr-login/login/",
                params={"code": request.code, "app_name": request.app_name}
            )
            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to login with WeChat")
            return response.json()

    async def 更新Token(self, request: TokenRefreshRequest, token: str = Depends(oauth2_scheme)) -> 返回_更新Token:
        # 调用用户管理微服务刷新Token
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/api/token/refresh-token/",
                json={"refresh_token": request.refresh_token},
                headers={"Authorization": f"Bearer {token}"}
            )
            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to refresh token")
            return response.json()

    async def 获取当前用户(self, token: str = Depends(oauth2_scheme)) -> Payload:
        # 将请求头中的Token传递给用户管理微服务
        if not token:
            raise HTTPException(
                status_code=401,
                detail="Failed to get current user")

        # header
        header = {"Authorization": f"Bearer {token}"}
        print(header)

        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/api/token/get-current-user/",
                headers=header
            )

        if response.status_code != 200:
            if response.json().get("detail", None) is not None:
                raise HTTPException(
                    status_code=response.status_code,
                    detail=response.json().get("detail"))
            else:
                raise HTTPException(
                    status_code=response.status_code,
                    detail=response.json() if response.content else "Failed to get current user")

        return response.json()

    async def 验证角色_from_header(
            self,
            info: 请求_验证角色_from_header,
            token: str = Depends(oauth2_scheme)) -> 返回_验证角色_from_header:
        # 调用用户管理微服务进行角色验证
        async with httpx.AsyncClient() as client:
            # url demo: http://localhost:8101/api/roles/role-auth/
            response = await client.post(
                f"{self.svc_user}/api/roles/role-auth/",
                json=请求_验证角色_from_header(role_name=info.role_name, app_name=info.app_name).model_dump_json(),
                headers={"Authorization": f"Bearer {token}"}
            )

            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to auth role")

            return response.json()

    async def 发送验证码(self, phone: str) -> dict:
        # 调用用户管理微服务发送验证码
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/api/account/send-verification-code/",
                params={"phone": phone}
            )
            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to send verification code")
            return response.json()

    async def 注册_手机(self, data: 请求_手机邮箱_注册) -> 返回_login:
        # 调用用户管理微服务进行手机注册
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/api/account/register-phone/",
                json=data.model_dump()
            )
            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to register with phone")
            return response.json()

    async def 登录_手机(self, data: 请求_手机邮箱_登录) -> 返回_login:
        # 调用用户管理微服务进行手机登录
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/api/account/login-phone/",
                json=data.model_dump()
            )
            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to login with phone")
            return response.json()

    async def 更换绑定手机号(self, new_phone: str, sms_code: str, token: str = Depends(oauth2_scheme)) -> 返回_login:
        # 调用用户管理微服务更换绑定手机号
        async with httpx.AsyncClient() as client:
            response = await client.post(
                f"{self.svc_user}/api/account/change-phone/",
                params={"new_phone": new_phone, "sms_code": sms_code},
                headers={"Authorization": f"Bearer {token}"}
            )
            if response.status_code != 200:
                if response.json().get("detail", None) is not None:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json().get("detail"))
                else:
                    raise HTTPException(
                        status_code=response.status_code,
                        detail=response.json() if response.content else "Failed to change phone number")
            return response.json()



from fastapi import FastAPI, Request
import uvicorn
import asyncio
import json
import base64
import datetime
import binascii
import datetime

app = FastAPI()
是否触发识别 = True


@app.post("/car/lcd")
async def 串口触发(data: Request):
    print("串口触发：",datetime.datetime.now(),"  ",await data.json())
    return "ok"


@app.post("/car/comet")
async def 轮询(data: Request):
    global 是否触发识别
    await asyncio.sleep(3)
    if 是否触发识别:
        return {"Response_AlarmInfoPlate": {"manualTrigger": "ok"}} # 回复 ok 进行手动触发
    else:
        return "ok"


@app.post("/car/getCarInfo")
async def 接受识别结果(data: Request):
    global 是否触发识别
    if not 是否触发识别:
        print("非手动触发结果")
        return "ok"
    else:
        是否触发识别 = False
    #
    res = await data.json()
    车牌号 = res["AlarmInfoPlate"]["result"]["PlateResult"]["license"]
    base64照片 = res["AlarmInfoPlate"]["result"]["PlateResult"]["imageFile"]
    拍摄时间dict = res["AlarmInfoPlate"]["result"]["PlateResult"]["timeStamp"]["Timeval"]
    拍摄时间str = "{decyear}{decmon:02d}{decday:02d}{dechour:02d}{decmin:02d}{decsec:02d}".format(**拍摄时间dict)
    车牌位置 = res["AlarmInfoPlate"]["result"]["PlateResult"]["location"]["RECT"]

    # 保存照片
    with open('imageFile/{}-{}.jpg'.format(拍摄时间str, 车牌号), 'wb') as imageFile:
        imgdata = base64.b64decode(base64照片)
        imageFile.write(imgdata)

    # BB88 AA 00                20150707 20150707 00 FF 33
    # BB88 AA 00 00413132333435 20150707 20150707 00 FF 33
    # 车牌号 00 41 31 32 33 34 35
    串口发送内容 = "BB88AA0000413132333435201507072015070700FF33"
    串口发送内容b64 = base64.b64encode(串口发送内容.encode("utf-8"))

    # 返回值
    if 车牌号 != "_无_":
        return {
            # Response_AlarmInfoPlate   Response_SerialData
            "Response_AlarmInfoPlate":
                {
                    "info": "ok",  # 回复ok开闸
                    "manualTigger": "ok",  # 回复 ok 进行手动触发
                    "is_pay": "true",
                    # 回复串口数据可以发送到相应串口
                    "serialData": [
                        # {"serialChannel": 0,
                        #  "data": 串口发送内容b64,
                        #  "dataLen": len(串口发送内容) / 2
                        #  },
                        {"serialChannel": 0,
                         "data": "AGT//+FBAQAACgAAAAAAAAAAABQLgAEAAAAAAAAAAAAAAAAAAABodHRwOi8vd3d3LmJhaWR1LmNvbQBwbGVhc2Ugc2NhbgDVJA==",
                         "dataLen": 73},

                    ],  # 数据1，可以有或者没有，收到后将发送到对应串口
                    # 白名单操作
                    # "white_list_operate":
                    #     {
                    #         "operate_type": 0,
                    #         "white_list_data": [
                    #             {"plate": 车牌号, "enable": 1, "need_alarm": 0,
                    #              "enable_time": datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                    #              "overdue_time": (datetime.datetime.now() + datetime.timedelta(days=1)).strftime(
                    #                  "%Y-%m-%d %H:%M:%S")}
                    #         ]
                    #     },
                },
        }
    else:
        data = "0064FFFFE13A0100000A0000000000000000000D0B80010000000000000000000000000000007777772E62616964752E636F6D00706C65617365207363616E00190D"
        dataB64 = "MDA2NEZGRkZFMTNBMDEwMDAwMEEwMDAwMDAwMDAwMDAwMDAwMDAwRDBCODAwMTAwMDAwMDAwMDAwMDAwMDAwMDAwMDAwMDAwMDAwMDc3Nzc3NzJFNjI2MTY5NjQ3NTJFNjM2RjZEMDA3MDZDNjU2MTczNjUyMDczNjM2MTZFMDAxOTBE"
        print("无牌车")

        return {"Response_SerialData": {
            "info": "ok",           # 回复ok开闸
            "channelNum":0,         # 回复开闸端口号，若无，则默认为0
            "manualTigger": "ok",   # 回复 ok 进行手动触发
            # 回复串口数据可以发送到相应串口
            "serialData": [
                {"serialChannel": 0, "data": "AGT//+FBAQAACgAAAAAAAAAAABQLgAEAAAAAAAAAAAAAAAAAAABodHRwOi8vd3d3LmJhaWR1LmNvbQBwbGVhc2Ugc2NhbgDVJA==","dataLen": 73},
            ],  # 数据1，可以有或者没有，收到后将发送到对应串口
        }}


if __name__ == '__main__':
    uvicorn.run(app, host="0.0.0.0", port=8001, debug=False)



from binascii import unhexlify
from crcmod import mkCrcFun
from app_tools_zxw.SDK_硬件读写.进制转换 import convert_int_16进制


def crc16(string):
    data = bytearray.fromhex(string)
    crc = 0xFFFF
    for pos in data:
        crc ^= pos
        for i in range(8):
            if ((crc & 1) != 0):
                crc >>= 1
                crc ^= 0xA001
            else:
                crc >>= 1
    return hex(((crc & 0xff) << 8) + (crc >> 8))[2:]


# CRC16/CCITT
def crc16_ccitt(s):
    crc16 = mkCrcFun(0x11021, rev=True, initCrc=0x0000, xorOut=0x0000)
    return get_crc_value(s, crc16)


# CRC16/CCITT-FALSE
def crc16_ccitt_false(s):
    crc16 = mkCrcFun(0x11021, rev=False, initCrc=0xFFFF, xorOut=0x0000)
    return get_crc_value(s, crc16)


# CRC16/IBM
def crc16_ibm(s):
    crc16 = mkCrcFun(0x18005, rev=True, initCrc=0x0000, xorOut=0x0000)
    x = get_crc_value(s, crc16)
    # x_reverse = x[3:] + x[0:2]
    return x


# CRC16/MODBUS
def crc16_modbus(s):
    crc16 = mkCrcFun(0x18005, rev=True, initCrc=0xFFFF, xorOut=0x0000)
    return get_crc_value(s, crc16)


# CRC16/XMODEM
def crc16_xmodem(s):
    crc16 = mkCrcFun(0x11021, rev=False, initCrc=0x0000, xorOut=0x0000)
    return get_crc_value(s, crc16)


# common func
def get_crc_value(s, crc16):
    data = s.replace(' ', '')
    crc_out = hex(crc16(unhexlify(data))).upper()
    str_list = list(crc_out)
    if len(str_list) == 5:
        str_list.insert(2, '0')  # 位数不足补0
    crc_data = ''.join(str_list[2:])
    return crc_data[:2] + ' ' + crc_data[2:]


if __name__ == '__main__':
    s0 = crc16("6001")
    s1 = crc16_ccitt("6001")
    s2 = crc16_ccitt_false("6001")
    s3 = crc16_modbus("6001")
    s4 = crc16_xmodem("6001")
    print('crc16: ' + s0)
    print('crc16_ccitt: ' + s1)
    print('crc16_ccitt_false: ' + s2)
    print('crc16_modbus: ' + s3)
    print('crc16_xmodem: ' + s4)




import asyncio
from typing import NoReturn
from datetime import datetime
from app_tools_zxw.SDK_硬件读写.进制转换 import *
from app_tools_zxw.SDK_硬件读写.串口_网口_通讯封装.串口读写.app应用级封装 import serial串口读写设备

"""
硬件设置：
    单张过滤：5秒
    rs232输出
"""


class readUHF卡(serial串口读写设备):
    # 覆写 - 参数配置
    波特率 = 115200
    读取字节长度 = 25

    # 覆写 - 定义返回数据格式（re）
    def _接受校验(self, d: str) -> bool:
        # 1B 39 01 FF 12 00 10 D4 30 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 F4
        # 1b3901ff120010d430000000000000000000000000000000f4
        d = d.upper()
        #
        验证1 = len(d) == self.读取字节长度 * 2
        if not 验证1:
            return False
        # 结构/命令 验证
        验证2 = d[:6] == "1B3901"
        验证3 = d[12:14] == "10"
        # 传输正确性验证:BCC校验
        ...
        return 验证1 and 验证2 and 验证3

    # 覆写 - 数据提取与转换
    def _提取数据(self, origin16Data: str):
        # 16进制数据转换
        data16 = origin16Data[20: 44]
        print(f"data16={len(data16), data16}")
        finalData = convert_16进制_int(data16)
        print(datetime.now(), "  读取到的卡号 = ", finalData)
        print("__________________________________________")
        return finalData


if __name__ == '__main__':
    # 读卡
    async def getdata(x: str) -> NoReturn:
        print("接受卡号 :", x)


    async def 转ws格式(some: str):
        return some


    com = readUHF卡(端口="/dev/tty.usbserial-CECRb189553")
    print("执行完毕")
    asyncio.run(com.持续get数据并转发(getdata, 转ws格式))



# from config import wx_APP_ID, wx_MCH_ID, wx_SECRET, wx_NONCE_STR, wx_KEY, PAYMENT_NOTIFY_URL, REFUND_NOTIFY_URL
from app_tools_zxw.configs import WeChatPay


class MiniAppsConfig:
    APP_ID = WeChatPay.APP_ID
    MCH_ID = WeChatPay.MCH_ID
    SECRET = WeChatPay.SECRET
    NONCE_STR = WeChatPay.NONCE_STR
    KEY = WeChatPay.KEY

    # 支付通知和退款通知，根据自己的后台接口填写
    PAYMENT_NOTIFY_URL = WeChatPay.PAYMENT_NOTIFY_URL_小程序
    REFUND_NOTIFY_URL = WeChatPay.REFUND_NOTIFY_URL_小程序

    # 微信退款需要用到的商户证书，没有配置的话请求退款会出错
    # 详情见：https://pay.weixin.qq.com/wiki/doc/api/wxa/wxa_api.php?chapter=4_3
    # 例如
    # CERT = '/appclient.pem'
    # CERT_KEY = '/appclient_key.pem'
    CERT = WeChatPay.CERT
    CERT_KEY = WeChatPay.CERT_KEY

    # 默认模式， 目前有 ignore 和 store， store则必须提供ORM模型类用来保存请求和响应记录
    # 目前之支持Django2.0以上版本
    DEFAULT_MODE = 'ignore'

    # 默认的ORM模型类，可以到用的时候再填
    DEFAULT_MODEL = None

    # 默认请求方法 post or get
    DEFAULT_METHOD = 'post'

    # 是否使用异步请求
    ASYNC = True

    # 如果微信接口不更新，下面的不用更改
    API_UNIFIED_ORDER = "https://api.mch.weixin.qq.com/pay/unifiedorder"
    API_ORDER_QUERY = "https://api.mch.weixin.qq.com/pay/orderquery"
    API_CLOSE_ORDER = "https://api.mch.weixin.qq.com/pay/closeorder"
    API_REFUND = "https://api.mch.weixin.qq.com/secapi/pay/refund"
    API_REFUND_QUERY = "https://api.mch.weixin.qq.com/pay/refundquery"



"""
# File       : api_服务器配置验证.py
# Time       ：2024/8/21 上午5:18
# Author     ：xuewei zhang
# Email      ：shuiheyangguang@gmail.com
# version    ：python 3.12
# Description：
"""
import hashlib
from fastapi import APIRouter, Response
from app_tools_zxw.Errors.api_errors import HTTPException_AppToolsSZXW
from config import WeChatPub

router = APIRouter(prefix="/wechat")


def check_signature(signature: str, timestamp: str, nonce: str) -> bool:
    """
    验证微信签名
    --------------------------
    signature 微信加密签名，signature结合了开发者填写的token参数和请求中的timestamp参数、nonce参数。
    timestamp 时间戳
    nonce 随机数
    echostr 随机字符串

    开发者通过检验signature对请求进行校验（下面有校验方式）。
    若确认此次GET请求来自微信服务器，请原样返回echostr参数内容，则接入生效，成为开发者成功，否则接入失败。
    加密/校验流程如下：
        1）将token、timestamp、nonce三个参数进行字典序排序
        2）将三个参数字符串拼接成一个字符串进行sha1加密
        3）开发者获得加密后的字符串可与signature对比，标识该请求来源于微信
    """
    token = WeChatPub.接口配置信息_Token
    # 将 token, timestamp, nonce 排序并拼接成字符串
    data = ''.join(sorted([token, timestamp, nonce]))
    # 使用 SHA1 算法加密
    sha1 = hashlib.sha1()
    sha1.update(data.encode('utf-8'))
    hashcode = sha1.hexdigest()
    # 判断 hashcode 是否与 signature 一致
    return hashcode == signature


@router.get("/verify")
async def wechat_verify(signature: str, timestamp: str, nonce: str, echostr: str):
    """微信公众号服务器验证"""
    # 校验签名
    if check_signature(signature, timestamp, nonce):
        return Response(content=echostr)
    else:
        raise HTTPException_AppToolsSZXW(error_code=1015,
                                         http_status_code=403,
                                         detail="Signature verification failed")


if __name__ == '__main__':
    signature = 'fc7017c67559cc25acffb96f302641ff8e8b5af1'
    echostr = '3894841337193693150'
    timestamp = '1724210780'
    nonce = '672323842'

    res = check_signature(signature, timestamp, nonce)
    print(res)



import asyncio
import datetime
from app_tools_zxw.SDK_硬件读写.串口_网口_通讯封装.串口读写.app应用级封装 import serial串口读写设备
from app_硬件通信.func_led出入指示 import LED文字指示系统

总读数次数上限 = 20
连续重量相等次数_判定阈值 = 2
录入系统的最低重量 = 1000  # 小轿车重1450KG
测量间隔 = 0.5
led文字指示 = LED文字指示系统()


# 新增方法
async def 提取车辆稳定后重量(read地磅数据: serial串口读写设备, 车辆方向is出: bool):
    前一秒重量 = 0
    重量相等的连续次数 = 0
    待选的实际重量 = []
    读数总次数 = 0
    while True:
        读数总次数 = 读数总次数 + 1
        # 1 读取地磅实时重量
        当前重量 = read地磅数据.单次get数据()
        print(f"{datetime.datetime.now()},当前重量 = {当前重量}")

        # 2 记录地磅稳定读数的持续时常
        if 当前重量 == 前一秒重量 and 当前重量 > 录入系统的最低重量:
            重量相等的连续次数 = 重量相等的连续次数 + 1
        else:
            重量相等的连续次数 = 0

        # 3 持续时常超过阈值：则记录为车辆实际重量
        if 重量相等的连续次数 >= 连续重量相等次数_判定阈值:
            待选的实际重量.append(当前重量)

        # 4 退出读数
        if len(待选的实际重量) > 0:
            最终真实重量 = max(待选的实际重量)
            return 最终真实重量

        # 5 异常退出
        else:
            if 读数总次数 > 总读数次数上限:
                return -99
            # 5 更新数据
            前一秒重量 = 当前重量

        # 6 测量间隔
        await asyncio.sleep(测量间隔)



import struct

async def send_data(session, data: str) -> int:
    """"""
    """ 发送数据，若成功，则返回发送胡数据长度"""
    # 计算传输头
    dataLen = len(data)

    headerList = [b'V', b'Z', 0, 0, dataLen >> 24 & 0xFF, dataLen >> 16 & 0xFF, dataLen >> 8 & 0xFF, dataLen & 0xFF]
    header = struct.pack("<ccBBBBBB", *headerList)
    # print("传输头：", headerList)

    # 传输数据
    await session.send(header)
    tmp = await session.send(data.encode())
    #
    return tmp


async def receive_data(session) -> dict:
    """"""
    """ 接受数据 """
    headerBytes = await session.recv(8)

    # 解析数据头
    header = struct.unpack("<ccBBBBBB", headerBytes)
    数据长度 = int(header[4] << 24) + int(header[5] << 16) + int(header[6] << 8) + int(header[7]) + 100
    # print("解析的数据头 = ", header, "数据长度 = ", 数据长度)

    # 读取数据
    if 数据长度 < 1024:
        data = await session.recv(数据长度)
    else:
        raise ValueError("接受长度超过1024请循环接受")


    return data


import re
import base64
from hashlib import md5
from Crypto.Cipher import AES
from app_tools_zxw.SDK_微信.SDK_微信支付.minipay.exceptions import OpenidError, TooManyArgumentError, ProductIdError
from app_tools_zxw.SDK_微信.SDK_微信支付.minipay.base import BaseMiniPay, BaseNotification
# from minipay.exceptions import (
#     OpenidError,
#     TooManyArgumentError,
#     ProductIdError
# )
# from minipay.base import BaseMiniPay, BaseNotification


class UnifiedOrder(BaseMiniPay):
    """
    doc: https://pay.weixin.qq.com/wiki/doc/api/wxa/wxa_api.php?chapter=9_1
    """

    def __init__(self, out_trade_no, body, total_fee,
                 device_info=None, detail=None, attach=None, fee_type=None,
                 time_start=None, time_expire=None, goods_tag=None, limit_pay=None,
                 openid=None, spbill_create_ip=None, trade_type=None, notify_url=None,
                 product_id=None, **kwargs):
        super(UnifiedOrder, self).__init__(**kwargs)
        self.target = kwargs.get('target') or self.config['api_unified_order']
        self.notify_url = notify_url or self.config['payment_notify_url']
        self.request_data = {
            'appid': self.config['app_id'],
            'mch_id': self.config['mch_id'],
            'device_info': device_info,
            'nonce_str': self.config['nonce_str'],
            'sign': None,
            'sign_type': None,
            'body': body,
            'detail': detail,
            'attach': attach,
            'out_trade_no': out_trade_no,
            'fee_type': fee_type,
            'total_fee': total_fee,
            'spbill_create_ip': spbill_create_ip or '0.0.0.0',
            'time_start': time_start,
            'time_expire': time_expire,
            'goods_tag': goods_tag,
            'notify_url': self.notify_url,
            'trade_type': trade_type or 'JSAPI',
            'product_id': product_id,
            'limit_pay': limit_pay,
            'openid': openid
        }

    def _decision_rules(self):
        if self.request_data['trade_type'] == 'JSAPI':
            if self.request_data['openid'] is None:
                raise OpenidError(
                    "request nimiapps api to unified order need 'openid' parameter when 'trade_type' is 'JSAPI'.")
        if self.request_data['trade_type'] == 'NATIVE':
            if self.request_data['product_id'] is None:
                raise ProductIdError(
                    "request nimiapps api to unified order need 'product_id' parameter when 'trade_type' is 'NATIVE'.")


class OrderQuery(BaseMiniPay):
    def __init__(self, out_trade_no=None, transaction_id=None, **kwargs):
        super(OrderQuery, self).__init__(**kwargs)
        self.target = kwargs.get('target') or self.config['api_order_query']
        self.request_data = {
            'appid': self.config['app_id'],
            'mch_id': self.config['mch_id'],
            'nonce_str': self.config['nonce_str'],
            'transaction_id': transaction_id,
            'out_trade_no': out_trade_no,
            'sign': None,
            'sign_type': None
        }

    def _decision_rules(self):
        if self.request_data['transaction_id'] is None and self.request_data['out_trade_no'] is None:
            raise TypeError("You must choose one from transaction_id or out_trade_no.")
        if self.request_data['transaction_id'] and self.request_data['out_trade_no']:
            raise TooManyArgumentError(
                "You can only choose one from transaction_id or out_trade_no.")


class CloseOrder(BaseMiniPay):
    def __init__(self, out_trade_no, **kwargs):
        super(CloseOrder, self).__init__(**kwargs)
        self.target = kwargs.get('target') or self.config['api_close_order']
        self.request_data = {
            'appid': self.config['app_id'],
            'mch_id': self.config['mch_id'],
            'out_trade_no': out_trade_no,
            'nonce_str': self.config['nonce_str'],
            'sign': None,
            'sign_type': None
        }


class Refund(BaseMiniPay):
    def __init__(self, out_refund_no, total_fee, refund_fee, notify_url=None,
                 refund_desc=None, transaction_id=None, out_trade_no=None,
                 refund_account=None, **kwargs):
        super(Refund, self).__init__(**kwargs)
        self.target = kwargs.get('target') or self.config['api_refund']
        self.notify_url = notify_url or self.config['refund_notify_url']
        self.options.setdefault(
            'cert', (self.config['cert'], self.config['cert_key'])
        )
        self.request_data = {
            'appid': self.config['app_id'],
            'mch_id': self.config['mch_id'],
            'nonce_str': self.config['nonce_str'],
            'sign': None,
            'sign_type': None,
            'transaction_id': transaction_id,
            'out_trade_no': out_trade_no,
            'out_refund_no': out_refund_no,
            'total_fee': total_fee,
            'refund_fee': refund_fee,
            'refund_desc': refund_desc,
            'refund_account': refund_account,
            'notify_url': self.notify_url
        }

    def _decision_rules(self):
        if self.request_data['transaction_id'] is None and self.request_data['out_refund_no'] is None:
            raise TypeError('You must choose one from transaction_id and out_trade_no.')
        if self.request_data['transaction_id'] and self.request_data['out_trade_no']:
            raise TooManyArgumentError("You can only choose one from transaction_id and out_trade_no.")


class RefundQuery(BaseMiniPay):
    """
    doc: https://pay.weixin.qq.com/wiki/doc/api/wxa/wxa_api.php?chapter=9_5
    """

    def __init__(self, transaction_id=None, out_trade_no=None, out_refund_no=None,
                 refund_id=None, offset=None, **kwargs):
        super(RefundQuery, self).__init__(**kwargs)
        self.target = kwargs.get('target') or self.config['api_refund_query']
        self.request_data = {
            'appid': self.config['app_id'],
            'mch_id': self.config['mch_id'],
            'nonce_str': self.config['nonce_str'],
            'sign': None,
            'sign_type': None,
            'transaction_id': transaction_id,
            'out_trade_no': out_trade_no,
            'out_refund_no': out_refund_no,
            'refund_id': refund_id,
            'offset': offset
        }

    def _decision_rules(self):
        if self.request_data['transaction_id'] is None and \
                self.request_data['out_refund_no'] is None and \
                self.request_data['out_trade_no'] is None and \
                self.request_data['refund_id'] is None:
            raise TypeError(
                "You must choose one from 'transaction_id','out_refund_no','out_trade_no', 'refund_id")

        metex_args = [
            self.request_data['transaction_id'],
            self.request_data['out_refund_no'],
            self.request_data['out_trade_no'],
            self.request_data['refund_id']
        ]
        count = 0
        for arg in metex_args:
            if arg:
                count += 1
            if count > 1:
                raise TooManyArgumentError(
                    "You can only choose one from 'transaction_id','out_refund_no','out_trade_no', 'refund_id"
                )


class PaymentNotification(BaseNotification):
    def __init__(self, data, **kwargs):
        super(PaymentNotification, self).__init__(data, **kwargs)


class RefundNotification(BaseNotification):
    """process refund notification process class"""
    def __init__(self, data, **kwargs):
        super(RefundNotification, self).__init__(data, **kwargs)

    def decrypt(self):
        """
         base64 decoding the encrypted string req_info to get b,
         md5 encoding b to get md5_key
        """
        b = base64.b64decode(self.response_data.get('req_info').encode('utf-8'))
        md5_key = md5(self.config['key'].encode('utf-8')).hexdigest()

        aes = AES.new(md5_key.encode('utf-8'), AES.MODE_ECB)
        encrypted_text = aes.decrypt(b)
        pattern = re.compile('<root>.*</root>', flags=re.S)
        encrypted_text = pattern.match(encrypted_text.decode('utf-8')).group()
        encrypted_dict = BaseMiniPay.xml_to_dict(encrypted_text, root='root')
        for k, v in encrypted_dict.items():
            self.response_data[k] = v
        self.response_data.pop('req_info')

    def _decision_rules(self):
        self.decrypt()

    def _verifysign(self):
        """退款通知无需验证签名"""
        return True




"""
# File       : ali_API装饰器.py
# Time       ：2022/8/17 05:35
# Author     ：xuewei zhang(张薛伟)
# Email      ：307080785@qq.com
# version    ：python 3.9
# Description：
"""
from typing import Any
from Tea.model import TeaModel
from functools import wraps
from pydantic import BaseModel
from decohints import decohints
import json


class ResBaseModel(BaseModel):
    Code: str
    Message: str
    RequestId: str = ""
    data: Any = ""


@decohints
def ali_api_process(ali_api_func):
    """
    阿里api调用函数返回值统一整理
    :param ali_api_func: 阿里api调用函数,需规定返回值为: (请求是否成功,返回数据)
            注意:请求是否成功 仅指api调用是否报错
    :return:
    """

    @wraps(ali_api_func)
    async def wrap_func(*args, **kwargs) -> ResBaseModel:
        res = await ali_api_func(*args, **kwargs)
        # res为TeaModel类型说明API请求成功
        if isinstance(res, TeaModel):
            # 整理 基本返回值 以外的数据,放在data字段里
            res: dict = res.body.to_map()
            data: dict = {}
            for key, value in res.items():
                if key not in ["Code", "Message", "RequestId"]:
                    data[key] = value
            return ResBaseModel(data=data, **res)
        # API请求发生错误
        elif isinstance(res, Exception):
            return ResBaseModel(**{"Code": "api响应错误error", "Message": str(kwargs.get("error")), "data": str(res)})

    return wrap_func


@decohints
def ali_api_process_旧版(ali_api_func):
    """
    阿里api调用函数返回值统一整理
    :param ali_api_func: 阿里api调用函数,需规定返回值为: (请求是否成功,返回数据)
            注意:请求是否成功 仅指api调用是否报错
    :return:
    """

    @wraps(ali_api_func)
    def wrap_func(*args, **kwargs) -> ResBaseModel:
        response: bytes = ali_api_func(*args, **kwargs)
        if isinstance(response, Exception):
            # API请求发生错误
            return ResBaseModel(**{"Code": "api响应错误error", "Message": str(kwargs.get("error")),"data": str(response)})
        else:
            res = json.loads(response.decode())
            return ResBaseModel(**res)

    return wrap_func



from aliyunsdkcore import client
from aliyunsdksts.request.v20150401 import AssumeRoleRequest
import json
import oss2
import time

'''
***************高级功能***************
###
### 使用STS进行临时授权
STS应用完整的示例代码请参见GitHub。
您可以通过STS（Security Token Service）进行临时授权访问。更多有关STS的内容请参见访问控制API参考（STS）中的简介。
关于账号及授权的详细信息请参见最佳实践中的STS临时授权访问。
首先您需要安装官方的Python STS客户端：pip install aliyun-python-sdk-sts
'''

accessKeyID = '见本地文件'
accessKeySecret = '见本地文件'
endpoint = 'oss-cn-shanghai.aliyuncs.com'  # 假设你的Bucket处于杭州区域
bucket名 = '见本地文件'
# role_arn是角色的资源名称。
RoleArn = 'acs:ram::1314717226629429:role/aliyunosstokengeneratorrole'
RoleSessionName = 'external-username'
DurationSeconds = 3600

# 阿里云主账号AccessKey拥有所有API的访问权限，风险很高。
# 强烈建议您创建并使用RAM账号进行API访问或日常运维，请登录 https://ram.console.aliyun.com 创建RAM账号。
clt = client.AcsClient(accessKeyID, accessKeySecret, 'cn-hangzhou')
req = AssumeRoleRequest.AssumeRoleRequest()
print(req.get_url('shanghai', accessKeyID, accessKeySecret))

# 设置返回值格式为JSON。
req.set_accept_format('json')
req.set_RoleArn(RoleArn)
req.set_RoleSessionName('session-name')
body = clt.do_action_with_exception(req)
# print(body)
# 使用RAM账号的AccessKeyId和AccessKeySecret向STS申请临时token。
token = json.loads(body)

# 使用临时token中的认证信息初始化StsAuth实例。
auth = oss2.StsAuth(token['Credentials']['AccessKeyId'],
                    token['Credentials']['AccessKeySecret'],
                    token['Credentials']['SecurityToken'])
# print(__auth.__auth())
# 使用StsAuth实例初始化存储空间。
bucket = oss2.Bucket(auth, endpoint, bucket名)

# 上传一个字符串。
bucket.put_object('object-name.txt', b'hello world')

# 上传图片，方法一
# with open('logo.jpg', 'rb') as f:
#     __bucket.put_object('logo11.jpg', f)

'''  
#######  使用签名Url上传文件  #######  
'''
# 生成请求Url
url = bucket.sign_url(method='PUT', key='logo444.jpg', expires=8)
url2 = bucket.sign_url(method='GET', key='xxxxx.jpg', expires=8)

print(url2)
# 上传图片，方法二
# __bucket.put_object_with_url_from_file(url,'logo.jpg')

# 上传图片，方法三
# time.sleep(10)
import requests

print(url)
res = requests.request('POST', url, data=open('logo.jpg', 'rb'))
print(res.headers)



import json
import numpy as np
from PIL import Image
from io import BytesIO
import httpx

'''
小程序码文档说明：
https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/qr-code/wxacode.getUnlimited.html
'''

baseUrl = 'https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token='

import sys


async def get小程序二维码(commerceID, tableID, wxAccessToken):
    postData = {
        'page': 'pages/index/index',
        'scene': 'id=%s&no=%s' % (str(commerceID), str(tableID)),  # 商户ID与桌号，-1代表外送订单
        'width': 280,  # 二维码大小
        'auto_color': False,  # 线条是否自动颜色
        'line_color': {"r": 0, "g": 0, "b": 0},  #
        'is_hyaline': False  # 底色是否透明
    }
    postUrl = baseUrl + wxAccessToken

    # 获取小程序二维码
    async with httpx.AsyncClient() as client:
        re = await client.post(postUrl, data=postData)
    图片二进制数据 = re.content

    # 读取成数组
    imageBytes = BytesIO(图片二进制数据)

    #
    print("图片大小 = ", sys.getsizeof(imageBytes) / 1024, "KB")

    return imageBytes


def 读取成图片对象(图片二进制数据):
    image = Image.open(BytesIO(图片二进制数据))
    array = np.array(image).tolist()
    return image


def 图片转化为Bytes(image):
    imageBytes = BytesIO()
    image.save(imageBytes, format='JPEG')
    return imageBytes


if __name__ == '__main__':
    data = get小程序二维码(commerceID='dads', tableID=-1, wxAccessToken="...")
    # plt.imshow(data)
    # plt.show()
